'\" t
.TH TCL3270 1 "19 June 2000"
.SH NAME
tcl3270 \-
.SM IBM
host access tool
.SH SYNOPSIS
.B tcl3270
.RI [ script ]
.RI [ options ]
.RI [ host ]
[\-\-
.IR script-arg ...]
.LP
.B tcl3270
opens a telnet connection to an
.SM IBM
host, then allows a tcl script to run the host login session.  It
is derived from
.IR x3270 ,
an X-windows IBM 3270 emulator.
It supports both the "standard" 3270 TELNET protocol and the TN3270E protocol,
except for printers, and supports
.B IND$FILE
file transfer.
.LP
The full syntax for
.I host
is:
.RS
.RI [ prefix :]...[ LUname @] hostname [: port ]
.RE
Prepending a
.B p:
onto
.I hostname
causes the connection to go through the
.I telnet-passthru
service rather than directly to the host.
See PASSTHRU below.
.LP
Prepending an
.B s:
onto
.I hostname
removes the "extended data stream" option reported to the host.
See
.B \-tn
below for further information.
.LP
A specific LU name to use may be specified by prepending it to the
.I hostname
with an
.RB `` @ ''.
.LP
The port to connect to defaults to
.BR telnet .
This can be overridden with the
.RI \- port
option, or by appending a
.I port
to the
.I hostname
with a colon
.RB `` : ''.
(For compatability with previous versions of
.I x3270
and with
.IR tn3270 ,
the
.I port
may also be specified as a second, separate argument.)
.SH OPTIONS
.B tcl3270
understands the following options:
.TP
.B \-apl
Sets up
.SM APL
mode.
Also see
.SM "APL SUPPORT"
below.
.TP
.BI \-charset " name"
Specifies an
.SM EBCDIC
national character set.
See
.SM CHARACTER SETS
below.
.TP
.BI \-clear " toggle"
Sets the initial value of
.I toggle
to
.B false.
The list of toggle names is under
.SM TOGGLES
below.
.TP
.BI \-model " name"
The model of 3270 display to be emulated.
The model name is in three parts, any of which may be omitted:
.IP
The first part is the
.BR "base model" ,
which is either
.B 3278
or
.BR 3279 .
.B 3278
specifies a monochrome 3270 display;
.B 3279
specifies a color 3270 display.
.IP
The second part is the
.BR "model number" ,
which specifies the number of rows and columns.
Model 4 is the default.
.PP
.RS
.TS
center;
c c c .
Model Number	Columns	Rows
_
2	80	24
3	80	30
4	80	43
5	132	27
.TE
.RE
.IP
Note: Technically, there is no such 3270 display as a 3279-4 or 3279-5, but
most hosts seem to work with them anyway.
.IP
The third part specifies the Extended 3270 Data Stream, and is given as
.B \-E.
It signals the host that the 3270 display is capable of displaying
extended field attributes, and supports structured fields and query replies.
A 3279 always uses the Extended Data Stream (whether or not
.B \-E
is specified); for a 3278 it is optional.
.IP
The default model is
.B 3279\-4\-E.
.TP
.BI \-oversize " cols" x rows
Makes the screen larger than the default for the chosen model number.
This option has effect only in combination with extended data stream support
(the
.B \-E
suffix on the model), and only if the host supports the Query Reply structured
field.
The number of columns multiplied by the number of rows must not exceed
16383 (3fff hex), the limit of 14-bit 3270 buffer addressing.
.TP
.BI \-port " n"
Specifies a different
.SM TCP
port to connect to.
.I n
can be a name from
.B /etc/services
like
.BR telnet ,
or a number.
This option changes the default port number used for all connections.
(The positional parameter affects only the initial connection.)
.TP
.BI \-set " toggle"
Sets the initial value of
.I toggle
to
.B true.
The list of toggle names is under
.SM TOGGLES
below.
.TP
.BI \-tn " name"
Specifies the terminal name to be transmitted over the telnet connection.
The default name is
.BI IBM\- model_name,
for example,
.BR IBM\-3279\-4\-E .
.IP
Some hosts are confused by the
.B \-E
suffix on the terminal name, and will ignore the extra screen area on
models 3, 4 and 5.
Prepending an
.B s:
onto the hostname removes the
.B \-E
from the terminal name when connecting to such hosts.
.TP
.B \-trace
Turns on data stream and event (script command) tracing.
.TP
.B \-\-
Terminates the list of
.B tcl3270
options.  Whatever follows will be available to the script in the
.B $argv
tcl variable.
.SH "CHARACTER SETS"
The
.B \-charset
option controls the
.SM EBCDIC
national character set used by
.B tcl3270.
Available sets include:
.PP
.TS
center;
l c
lfB c.
Charset Name	Q121 Code
_
bracket	\-
us-intl	01
german	03
finnish	09
uk	22
norwegian	23
french	30
hebrew	\-
icelandic	\-
belgian	\-
.TE
.PP
The default character set is
.BR bracket ,
which is useful for common
.SM IBM
hosts which use
.SM EBCDIC
codes 0xAD and 0xBD for the `[' and `]' characters, respectively.
.SH "ANSI MODE"
Some hosts use an
.SM ASCII
front-end to do initial login negotiation, then later switch to 3270 mode.
.B tcl3270
will emulate an
.SM ANSI
X.64 terminal until the host places it in 3270 mode (telnet
.SM BINARY
and
.SM "SEND EOR"
modes).
This allows
.B tcl3270
to be used to script standard TELNET sessions as well.
.PP
If the host later negotiates to stop functioning in 3270 mode,
.B tcl3270
will return to
.SM ANSI
emulation.
.PP
When emulating an
.SM ANSI
terminal,
.B tcl3270
supports both character-at-a-time mode and line mode operation.
When in line mode, the special characters and operational characteristics are
defined as follows:
.PP
.TS
center;
l c.
Mode/Character	Setting
_
Translate CR to NL	true
Translate NL to CR	false
Erase previous character	^?
Erase entire line	^U
Erase previous word	^W
Redisplay line	^R
Ignore special meaning of next character	^V
Interrupt	^C
Quit	^\\\\ 
End of file	^D
.TE
.SH TOGGLES
.B tcl3270
has a number of configurable modes which may be selected by the -set and -clear options.
.TP
.B monoCase
If set,
.B tcl3270
operates in uppercase-only mode.
.TP
.B blankFill
If set,
.B tcl3270
behaves in some un-3270-like ways.
First, when a character is typed into a field, all nulls in the field to the
left of that character are changed to blanks.
This eliminates a common 3270 data-entry surprise.
Second, in insert mode, trailing blanks in a field are treated like nulls,
eliminating the annoying ``lock-up'' that often occurs when inserting into an
field with (apparent) space at the end.
.TP
.B lineWrap
If set, the
.SM ANSI
terminal emulator automatically assumes a
.SM NEWLINE
character when it reaches the end of a line.
.SH COMMANDS
.B tcl3270
supports the following additional tcl commands:
.PP
.TS
l l
.
Ascii	return entire screen contents in ASCII
Ascii \fIlen\fP	return screen contents at cursor, in ASCII
Ascii \fIrow col len\fP	return screen contents in ASCII
Ascii \fIrow col rows cols\fP	return screen region in ASCII
AsciiField	return current field in ASCII
Attn	attention key
BackSpace	move cursor left (or send ASCII BS)
BackTab	tab to start of previous input field
CircumNot	input "^" in ANSI mode, or "notsign" in 3270 mode
Clear	clear screen
Connect \fIhost\fP	connect to \fIhost\fP
CursorSelect	Cursor Select AID
Cut	erase selected text
Delete	delete character under cursor (or send ASCII DEL)
DeleteField	delete the entire field
DeleteWord	delete the current or previous word
Disconnect	disconnect from the host
Down	move cursor down
Dup	duplicate field
Ebcdic	return entire screen contents in EBCDIC
Ebcdic \fIlen\fP	return screen contents at cursor, in EBCDIC
Ebcdic \fIrow col len\fP	return screen contents in EBCDIC
Ebcdic \fIrow col rows cols\fP	return screen region in EBCDIC
EbcdicField	return current field in EBCDIC
Enter	Enter AID (or send ASCII CR)
Erase	erase previous character (or send ASCII BS)
EraseEOF	erase to end of current field
EraseInput	erase all input fields
FieldEnd	move cursor to end of field
FieldExit	clear to end of field and skip to next (5250 emulation)
FieldMark	mark field
HexString \fIhex_digits\fP	insert control-character string
Home	move cursor to first input field
Insert	set insert mode
Key \fIkeysym\fP	insert key \fIkeysym\fP
Left	move cursor left
Left2	move cursor left 2 positions
MoveCursor \fIrow col\fP	move cursor
MonoCase	toggle uppercase-only mode
Newline	move cursor to first field on next line (or send ASCII LF)
NextWord	move cursor to next word
PA \fIn\fP	Program Attention AID (\fIn\fP from 1 to 3)
PF \fIn\fP	Program Function AID (\fIn\fP from 1 to 24)
PreviousWord	move cursor to previous word
Quit	exit \fBtcl3270\fP
Reset	reset locked keyboard
Right	move cursor right
Right2	move cursor right 2 positions
Status	report connection status
String \fIstring\fP	insert string
SysReq	System Request AID
Tab	move cursor to next input field
ToggleInsert	toggle insert mode
Transfer \fIoption\fP=\fIvalue\fP...	file transfer
Up	move cursor up
Wait	wait for valid input field
Wait 3270	wait for 3270 mode
Wait ansi	wait for NVT mode
Wait Disconnect	wait for host to disconnect
Wait Output	wait for more host output
.TE
.SH "STRING COMMAND"
The arguments to String are one or more double-quoted strings which are
inserted directly as if typed.
The C backslash conventions are honored as follows.
(Entries marked * mean that after sending the
.SM AID
code to the host,
.B tcl3270
will wait for the host to unlock the keyboard before further processing the
string.)
.RS 1i
.TS
l l.
\eb	Left
\ef	Clear*
\en	Enter*
\e\epa\fIn\fP	PA key \fIn\fP*
\e\epf\fInn\fP	PF key \fInn\fP*
\er	Newline
\et	Tab
.TE
.RE
.IP
.B Note:
The strings are in
.SM ASCII
and converted to
.SM EBCDIC,
so beware of inserting
control codes.
.PP
There is also an alternate form of the String command, HexString, which is
used to enter non-printing data.
The argument to HexString is a string of hexadecimal digits, two per
character.  A leading 0x or 0X is optional.
In 3270 mode, the hexadecimal data represent EBCDIC characters, which are
entered into the current field.
In ANSI mode, the hexadecimal data represent ASCII characters, which are sent
directly to the host.
.SH "STATUS COMMAND"
The result of the Status command consists of 11 blank-separated fields:
.TP
Keyboard State
If the keyboard is unlocked, the letter
.BR U .
If the keyboard is locked waiting for a response from the host, or if not
connected to a host, the letter
.BR L .
If the keyboard is locked because of an operator error (field overflow,
protected field, etc.), the letter
.BR E .
.TP
Screen Formatting
If the screen is formatted, the letter
.BR F .
If unformatted or in
.SM ANSI
mode,
the letter
.BR U .
.TP
Field Protection
If the field containing the cursor is protected, the letter
.BR P .
If unprotected or unformatted, the letter
.BR U .
.TP
Connection State
If connected to a host, the string
.BI C( hostname ).
Otherwise, the letter
.B N .
.TP
Emulator Mode
If connected in 3270 mode, the letter
.BR I .
If connected in ANSI line mode, the letter
.BR L .
If connected in ANSI character mode, the letter
.BR C .
If not connected, the letter
.BR N .
.TP
Model Number (2-5)
.TP
Number of Rows
The current number of rows defined on the screen.
The host can request that
.B tcl3270
use a 24x80 screen, so this number may be smaller than the maximum number of
rows possible with the current model.
.TP
Number of Columns
The current number of columns defined on the screen, subject to the same
difference for rows, above.
.TP
Cursor Row
The current cursor row (zero-origin).
.TP
Cursor Column
The current cursor column (zero-origin).
.SH "SCREEN CONTENTS COMMANDS"
There are four commands to return the current virtual screen contents.
The
.B Ascii
and
.B AsciiField
commands return a region of the screen as an ASCII text.
The
.B Ebcdic
and
.B EbcdicField
commands return a region of the screen as EBCDIC bytes, with each character
represented as a 2-digit hexadecimal string, prefixed by
.BR 0x .
.PP
The
.B AsciiField
and
.B EbcdicField
commands return the contents of the field on the screen which contains the
cursor.
.PP
The
.B Ascii
and
.B Ebcdic
commands return abitrary regions of the virtual screen.
They take the same arguments:
.RS
.PP
If no arguments are given, then entire screen contents are returned.
.PP
If one argument is given
.RI ( len ),
the region of the screen starting at the cursor,
.I len
bytes long is returned.
.PP
If three arguments are given
.RI ( row ,
.IR col ,
.IR len ),
the region of the screen starting at zero-origin coordinates
.RI ( row ,
.IR col ),
length
.I len
bytes, is returned.
.PP
If four arguments are given
.RI ( row ,
.IR col ,
.IR rows ,
.IR cols ),
then a rectangular region is returned, starting at zero-origin coordinates
.RI ( row ,
.IR col ),
width
.IR rows ,
length
.IR cols .
.RE
.SH "TRANSFER COMMAND"
The Transfer command is used to invoke IND$FILE file transfer.
Note that this command requires that the
.B IND$FILE
program be installed on the IBM host, and that the 3270 cursor
be located in a field that will accept a TSO or VM/CMS command.
.PP
Because of the complexity and number of options for file transfer, the
parameters to the Transfer command take the unique form of
.IR option = value ,
and can appear in any order.
The options are:
.PP
.TS
l c l l.
Option	Required?	Default	Other Values
_
Direction	No	send	receive
HostFile	Yes
LocalFile	Yes
Host	No	tso	vm
Mode	No	ascii	binary
Cr	No	remove	add, keep
Exist	No	keep	replace, append
Recfm	No		fixed, variable, undefined
Lrecl	No
Blksize	No
Allocation	No		tracks, cylinders, avblock
PrimarySpace	No
SecondarySpace	No
.TE
.PP
The option details are as follows.
.TP
.B Direction
.B send
(the default) to send a file to the host,
.B receive
to receive a file from the host.
.TP
.B HostFile
The name of the file on the host.
.TP
.B LocalFile
The name of the file on the local workstation.
.TP
.B Host
The type of host (which dictates the form of the
.B IND$FILE
command):
.B tso
(the default)
or
.BR vm .
.TP
.B Mode
Use
.B ascii
(the default) for a text file, which will be translated between EBCDCIC and
ASCII as necessary.
Use
.B binary
for non-text files.
.TP
.B Cr
Controls how Newline characters are handled when transferring
.B Mode=ascii
files.
.B remove
(the default) strips Newline characters in local files before transferring
them to the host.
.B add
adds Newline characters to each host file record before transferring it to
the local workstation.
.B keep
preserves Newline characters when transferring a local file to the host.
.TP
.B Exist
Controls what happens when the destination file already exists.
.B keep
(the default) preserves the file, causing the Transfer command to fail.
.B replace
overwrites the destination file with the source file.
.B append
appends the source file to the destination file.
.TP
.B Recfm
Controls the record format of files created on the host.
.B fixed
creates a file with fixed-length records.
.B variable
creates a file with variable-length records.
.B undefined
creates a file with undefined-length records (TSO hosts only).
The
.B Lrecl
option controls the record length or maximum record length for
.B Recfm=fixed
and
.B Recfm=variable
files, respectively.
.TP
.B Lrecl
Specifies the record length (or maximum record length) for files created on
the host.
.TP
.B Blksize
Specifies the block size for files created on the host.  (TSO hosts only.)
.TP
.B Allocation
Specifies the units for the TSO host
.B PrimarySpace
and
.B SecondarySpace
options:
.BR tracks ,
.B cylinders
or
.BR avblock .
.TP
.B PrimarySpace
Primary allocation for a file created on a TSO host.
The units are given by the
.B Allocation
option.
.TP
.B SecondarySpace
Secondary allocation for a file created on a TSO host.
The units are given by the
.B Allocation
option.
.SH "APL SUPPORT"
.B tcl3270
supports the full
.SM APL2
character set and the entry of
.SM APL
characters with the Key command.
.PP
The complete list of special
.SM APL
keysyms is as follows.  Entries marked with an asterisk (*) represent
simple aliases for standard \s-1EBCDIC\s+1 characters.
.PP
.RS
.TS
l c l.
\s-1APL\s+1 Symbol	Hex	tcl3270 Keysym
_
A underbar	41	apl_Aunderbar
alpha	B0	apl_alpha
B underbar	42	apl_Bunderbar
bar	60*	apl_bar
C underbar	43	apl_Cunderbar
circle	9D	apl_circle
circle bar	ED	apl_circlebar
circle slope	CF	apl_circleslope
circle star	FD	apl_circlestar
circle stile	CD	apl_circlestile
colon	7A*	apl_colon
comma	6B*	apl_comma
D underbar	44	apl_Dunderbar
del	BA	apl_del
del stile	DC	apl_delstile
del tilde	FB	apl_deltilde
delta	BB	apl_delta
delta stile	DD	apl_deltastile
delta underbar	FC	apl_deltaunderbar
diamond	70	apl_diamond
dieresis	72	apl_dieresis
dieresis dot	EC	apl_dieresisdot
divide	B8	apl_divide
dot	4B*	apl_dot
down arrow	8B	apl_downarrow
down caret	78	apl_downcaret
down caret tilde	CB	apl_downcarettilde
down shoe	AB	apl_downshoe
down stile	8E	apl_downstile
down tack	AC	apl_downtack
down tack jot	FE	apl_downtackjot
down tack up tack	DA	apl_downtackuptack
E underbar	45	apl_Eunderbar
epsilon	B1	apl_epsilon
epsilon underbar	75	apl_epsilonunderbar
equal	7E*	apl_equal
equal underbar	E1	apl_equalunderbar
F underbar	46	apl_Funderbar
G underbar	47	apl_Gunderbar
greater	6E*	apl_greater
H underbar	48	apl_Hunderbar
I underbar	49	apl_Iunderbar
iota	B2	apl_iota
iota underbar	74	apl_iotaunderbar
J underbar	51	apl_Junderbar
jot	AF	apl_jot
K underbar	52	apl_Kunderbar
L underbar	53	apl_Lunderbar
left arrow	9F	apl_leftarrow
left bracket	AD	apl_leftbracket
left paren	4D*	apl_leftparen
left shoe	9B	apl_leftshoe
less	4C*	apl_less
M underbar	54	apl_Munderbar
N underbar	55	apl_Nunderbar
not equal	BE	apl_notequal
not greater	8C	apl_notgreater
not less	AE	apl_notless
O underbar	56	apl_Ounderbar
omega	B4	apl_omega
overbar	A0	apl_overbar
P underbar	57	apl_Punderbar
plus	4E*	apl_plus
Q underbar	58	apl_Qunderbar
quad	90	apl_quad
quad divide	EE	apl_quaddivide
quad jot	73	apl_quadjot
quad quote	DE	apl_quadquote
quad slope	CE	apl_quadslope
query	6F*	apl_query
quote	7D*	apl_quote
quote dot	DB	apl_quotedot
R underbar	59	apl_Runderbar
rho	B3	apl_rho
right arrow	8F	apl_rightarrow
right bracket	BD	apl_rightbracket
right paren	5D*	apl_rightparen
right shoe	9A	apl_rightshoe
S underbar	62	apl_Sunderbar
semicolon	5E*	apl_semicolon
slash	61*	apl_slash
slash bar	EA	apl_slashbar
slope	B7	apl_slope
slope bar	EB	apl_slopebar
squad	CC	apl_squad
star	5C*	apl_star
stile	BF	apl_stile
T underbar	63	apl_Tunderbar
tilde	80	apl_tilde
times	B6	apl_times
U underbar	64	apl_Uunderbar
underbar	6D*	apl_underbar
up arrow	8A	apl_uparrow
up caret	71	apl_upcaret
up caret tilde	CA	apl_upcarettilde
up shoe	AA	apl_upshoe
up shoe jot	DF	apl_upshoejot
up stile	8D	apl_upstile
up tack	BC	apl_uptack
up tack jot	EF	apl_uptackjot
V underbar	65	apl_Vunderbar
W underbar	66	apl_Wunderbar
X underbar	67	apl_Xunderbar
Y underbar	68	apl_Yunderbar
Z underbar	69	apl_Zunderbar
.TE
.RE
.SH PASSTHRU
.B tcl3270
supports the Sun
.I telnet-passthru
service provided by the
.I in.telnet-gw
server.
This allows outbound telnet connections through a firewall machine.
When a
.B p:
is prepended to a hostname,
.B tcl3270
acts much like the
.IR itelnet(1)
command.
It contacts the machine named
.B internet-gateway
at the port defined in
.B /etc/services
as
.B telnet-passthru
(which defaults to 3514).
It then passes the requested hostname and port to the
.B in.telnet-gw
server.
.SH "SEE ALSO"
tcl(1), telnet(1), tn3270(1), ibm_hosts(5), x3270(1)
.br
X Toolkit Intrinsics
.br
Data Stream Programmer's Reference, IBM GA23\-0059
.br
Character Set Reference, IBM GA27\-3831
.br
RFC 1576, TN3270 Current Practices
.br
RFC 1646, TN3270 Extensions for LUname and Printer Selection
.br
RFC 2355, TN3270 Enhancements
.SH COPYRIGHTS
.LP
Modifications Copyright 1993, 1994, 1995, 1996, 1997, 2000 by Paul Mattes.
.br
Original X11 Port Copyright 1990 by Jeff Sparkes.
.RS
Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation.
.RE
Copyright 1989 by Georgia Tech Research Corporation, Atlanta, GA 30332.
.RS
All Rights Reserved.  GTRC hereby grants public use of this software.
Derivative works based on this software must incorporate this copyright
notice.
.RE
5250 Emulation Code copyright Minolta (Schweiz) AG, Beat Rubischon.
.SH VERSION
tcl3270 3.2.8
